import { useAjax } from "@/Hook/useAjax"
import TableData from "@/pages/launchSystemNew/components/TableData"
import { delSysWarningRuleApi, getSysWarningRuleListApi, RuleProps } from "@/services/adMonitor/earlyWarning"
import { Button, Drawer, Input, message, Space } from "antd"
import React, { useEffect, useState } from "react"
import AddEdit from "./addEdit"
import ExpandedRowTable from "./expandedRowTable"
import tableConfig from "./tableConfig"



/**
 * 监控预警
 * @returns 
 */
const EarlyWarning: React.FC = () => {

    /**************************/
    const [visible, setVisible] = useState<boolean>(false)
    const [addVisible, setAddVisible] = useState<boolean>(false)
    const [queryParams, setQueryParams] = useState<{ ruleName?: string }>({})
    const [editData, setEditData] = useState<{ initialValues?: any }>({})

    const getSysWarningRuleList = useAjax((params) => getSysWarningRuleListApi(params), { formatResult: true })
    const delSysWarningRule = useAjax((params) => delSysWarningRuleApi(params))
    /**************************/

    // 获取列表
    useEffect(() => {
        getList()
    }, [])

    const getList = () => {
        getSysWarningRuleList.run(queryParams)
    }

    /** 修改 */
    const edit = (data: any) => {
        let newData = JSON.parse(JSON.stringify(data))
        newData.defaultRule = newData.defaultRule ? true : false
        newData.enable = newData.enable ? true : false
        newData.rules = newData.rules.map((item: { rule: RuleProps[] }) => {
            let field: string[] = []
            let data: any = {}
            item.rule.forEach(item => {
                let { field: oldField, ...newItem } = item
                field.push(oldField)
                data[oldField] = { ...newItem }
            })
            return {
                ...data,
                field
            }
        })

        setEditData({ initialValues: newData })
        setAddVisible(true)
    }
    /** 删除 */
    const del = (id: number) => {
        delSysWarningRule.run(id).then(res => {
            message.success('删除成功')
            getSysWarningRuleList.refresh()
        })
    }

    return <>
        <Button type="primary" onClick={() => { setVisible(true); setEditData({}) }}>监控预警</Button>

        {/* 预警列表 */}
        {visible && <Drawer
            title="监控预警"
            placement="right"
            onClose={() => setVisible(false)}
            visible={visible}
            width='800px'
        >
            <Space direction="vertical">
                <Space>
                    <Input placeholder="请输入规则名字" value={queryParams?.ruleName} onChange={(e) => setQueryParams({ ...queryParams, ruleName: e.target.value })} />
                    <Button type="primary" onClick={getList}>搜索</Button>
                    <Button type="primary" onClick={() => setAddVisible(true)}>设置预警</Button>
                </Space>

                <TableData
                    size="small"
                    isCard={false}
                    columns={() => tableConfig(edit, del)}
                    ajax={getSysWarningRuleList}
                    dataSource={getSysWarningRuleList?.data?.data}
                    loading={getSysWarningRuleList?.loading}
                    total={getSysWarningRuleList?.data?.data?.length}
                    gutter={[0, 10]}
                    leftChild={<Space direction='vertical'></Space>}
                    expandedRowRender={(data) => <ExpandedRowTable data={data}/>}
                />
            </Space>
        </Drawer>}

        {/* 设置预警 */}
        {addVisible && <AddEdit visible={addVisible} onChange={() => { setAddVisible(false); getSysWarningRuleList.refresh(); setEditData({}) }} onClose={() => { setAddVisible(false); setEditData({}) }} {...editData} />}
    </>
}

export default React.memo(EarlyWarning)